@@ -7,7 +7,7 @@ from time import time
 
 import requests
 
-from octopus import Octopus
+from octopus import Octopus, TornadoOctopus
 
 
 def main(repetitions, concurrency):
@@ -33,6 +33,62 @@ def main(repetitions, concurrency):
 
     urls_to_retrieve = [choice(urls) for i in range(repetitions)]
 
+    #requests_total_time = sequential_requests(repetitions, urls_to_retrieve)
+    otto_total_time = otto_requests(repetitions, concurrency, urls_to_retrieve)
+    otto_cached_total_time = otto_cached_requests(repetitions, concurrency, urls_to_retrieve)
+    tornado_pycurl_total_time = tornado_requests(repetitions, concurrency, urls_to_retrieve)
+    tornado_total_time = tornado_requests(repetitions, concurrency, urls_to_retrieve, ignore_pycurl=True)
+
+    message = "RESULTS"
+    print
+    print("=" * len(message))
+    print(message)
+    print("=" * len(message))
+    print
+
+    #print "[requests] Retrieving %d urls took %.2f seconds meaning %.2f urls/second." % (
+        #repetitions,
+        #requests_total_time,
+        #repetitions / requests_total_time
+    #)
+    #print
+
+    print "[octopus] Retrieving %d urls took %.2f seconds meaning %.2f urls/second." % (
+        repetitions,
+        otto_total_time,
+        repetitions / otto_total_time
+    )
+    print
+
+    print "[octopus] Retrieving %d urls with local in-memory caching took %.2f seconds meaning %.2f urls/second." % (
+        repetitions,
+        otto_cached_total_time,
+        repetitions / otto_cached_total_time
+    )
+    print
+
+    print "[octopus-tornado] Retrieving %d urls took %.2f seconds meaning %.2f urls/second." % (
+        repetitions,
+        tornado_total_time,
+        repetitions / tornado_total_time
+    )
+    print
+
+    print "[octopus-tornado-pycurl] Retrieving %d urls took %.2f seconds meaning %.2f urls/second." % (
+        repetitions,
+        tornado_pycurl_total_time,
+        repetitions / tornado_pycurl_total_time
+    )
+    print
+
+    #print "Overall, octopus was more than %.2f times faster than sequential requests." % (
+        #int(requests_total_time / otto_total_time)
+    #)
+
+    #print
+
+
+def sequential_requests(repetitions, urls_to_retrieve):
     message = "Retrieving URLs sequentially with Requests..."
     print
     print("=" * len(message))
@@ -49,8 +105,10 @@ def main(repetitions, concurrency):
         )
         assert requests.get(url).status_code == 200
 
-    requests_total_time = time() - start_time
+    return time() - start_time
 
+
+def otto_requests(repetitions, concurrency, urls_to_retrieve):
     message = "Retrieving URLs concurrently with Octopus..."
     print
     print("=" * len(message))
@@ -67,8 +125,10 @@ def main(repetitions, concurrency):
     otto.start()
     otto.wait(0)
 
-    otto_total_time = time() - start_time
+    return time() - start_time
 
+
+def otto_cached_requests(repetitions, concurrency, urls_to_retrieve):
     message = "Retrieving URLs concurrently with Octopus with caching enabled..."
     print
     print("=" * len(message))
@@ -84,41 +144,28 @@ def main(repetitions, concurrency):
     start_time = time()
     otto.wait(0)
 
-    otto_cached_total_time = time() - start_time
+    return time() - start_time
 
-    message = "RESULTS"
+
+def tornado_requests(repetitions, concurrency, urls_to_retrieve, ignore_pycurl=False):
+    message = "Retrieving URLs concurrently with TornadoOctopus (%s)..." % (
+        ignore_pycurl and "using SimpleHTTPClient" or "using pycurl"
+    )
     print
     print("=" * len(message))
     print(message)
     print("=" * len(message))
     print
 
-    print "[requests] Retrieving %d urls took %.2f seconds meaning %.2f urls/second." % (
-        repetitions,
-        requests_total_time,
-        repetitions / requests_total_time
-    )
-    print
+    otto = TornadoOctopus(concurrency=concurrency, cache=True, auto_start=True, ignore_pycurl=ignore_pycurl)
 
-    print "[octopus] Retrieving %d urls took %.2f seconds meaning %.2f urls/second." % (
-        repetitions,
-        otto_total_time,
-        repetitions / otto_total_time
-    )
-    print
+    for url in urls_to_retrieve:
+        otto.enqueue(url, handle_url_response)
 
-    print "[octopus] Retrieving %d urls with local in-memory caching took %.2f seconds meaning %.2f urls/second." % (
-        repetitions,
-        otto_cached_total_time,
-        repetitions / otto_cached_total_time
-    )
-    print
+    start_time = time()
+    otto.wait(0)
 
-    print "Overall, octopus was more than %.2f times faster than sequential requests." % (
-        int(requests_total_time / otto_total_time)
-    )
-
-    print
+    return time() - start_time
 
 
 def handle_url_response(url, response):
